class UnknownGitHubRepositoryException(Exception):

    def __init__(self, repositorySlug: str):

        self._repositorySlug: str = repositorySlug

    @property
    def repositorySlug(self) -> str:
        return self._repositorySlug




from logging import Logger
from logging import getLogger


from curses import initscr
from curses import noecho
from curses import cbreak
from curses import endwin
from curses import nocbreak
from curses import echo

from _curses import A_UNDERLINE

from versionoverlord.Common import SlugVersions

PACKAGE_VERSION_GAP: int = 2
TITLE_LINE_GAP:      int = 2


class DisplayVersions:
    def __init__(self):
        self.logger: Logger = getLogger(__name__)

        self._stdScreen = initscr()

    def displaySlugs(self, slugVersions: SlugVersions):

        self._initializeCurses()

        longest:      int = self._getLongest(slugVersions)
        lineNumber:   int = 0
        columnNumber: int = 0
        packageColumnNumber: int = longest + PACKAGE_VERSION_GAP

        self._stdScreen.addstr(lineNumber, columnNumber,        'Slug',    A_UNDERLINE)
        self._stdScreen.addstr(lineNumber, packageColumnNumber, 'Version', A_UNDERLINE)

        lineNumber += TITLE_LINE_GAP
        for slugVersion in slugVersions:

            self._stdScreen.addstr(lineNumber, columnNumber,        f'{slugVersion.slug}:')
            self._stdScreen.addstr(lineNumber, packageColumnNumber, f'{slugVersion.version}')
            lineNumber += 1

        self._stdScreen.addstr(lineNumber + 1, columnNumber, 'Press any key to continue...')
        # noinspection PyUnusedLocal
        s = self._stdScreen.getch()

        self._done()

    def _initializeCurses(self):
        noecho()
        cbreak()
        self._stdScreen.keypad(True)

    def _done(self):
        nocbreak()
        self._stdScreen.keypad(False)
        echo()
        endwin()

    def _getLongest(self, slugVersions: SlugVersions) -> int:
        longest: int = 0
        for slugVersion in slugVersions:
            if len(slugVersion.slug) > longest:
                longest = len(slugVersion.slug)

        return longest




from logging import Logger
from logging import getLogger

from os import sep as osSep

from pathlib import Path

from versionoverlord.Common import CIRCLE_CI_DIRECTORY
from versionoverlord.Common import CIRCLE_CI_YAML

from versionoverlord.IHandler import IHandler
from versionoverlord.Common import Packages


class HandleCircleCI(IHandler):

    def __init__(self, packages: Packages):

        self.logger: Logger = getLogger(__name__)

        super().__init__(packages)

        self._circleCIYAML: Path = Path(f'{self._projectsBase}{osSep}{self._projectDirectory}{osSep}{CIRCLE_CI_DIRECTORY}{osSep}{CIRCLE_CI_YAML}')

    @property
    def configurationExists(self) -> bool:
        return self._circleCIYAML.exists()

    def update(self):

        self._update(configurationFilePath=self._circleCIYAML)




from pathlib import Path

from click import ClickException
from click import command
from click import option
from click import secho
from click import version_option

from versionoverlord import __version__
from versionoverlord.Common import AdvancedSlugs
from versionoverlord.Common import CLISlugs
from versionoverlord.Common import extractCLISlugs

from versionoverlord.Common import setUpLogging

from versionoverlord.FileNameToSlugs import FileNameToSlugs
from versionoverlord.TemplateHandler import TemplateHandler
from versionoverlord.exceptions.NoGitHubAccessTokenException import NoGitHubAccessTokenException


# noinspection SpellCheckingInspection
@command()
@version_option(version=f'{__version__}', message='%(prog)s version %(version)s')
@option('--slugs',     '-s',  multiple=True, required=False, help='Create package update specification')
@option('--input-file', '-i', required=False,                help='Use input file for slug list')
def createSpecification(slugs: CLISlugs, input_file: str):
    """
    \b
    This command creates .csv specification file
    It uses the following environment variables:
    \b
        GITHUB_ACCESS_TOKEN – A personal GitHub access token necessary to read repository release information
        PROJECTS_BASE – The local directory where the python projects are based
        PROJECT – The name of the project;  It should be a directory name
    """

    try:
        if len(slugs) != 0:
            cliSlugs:        AdvancedSlugs   = extractCLISlugs(slugs=slugs)
            templateHandler: TemplateHandler = TemplateHandler(cliSlugs)

            templateHandler.createSpecification()
        elif input_file is not None:
            fqFileName: Path = Path(input_file)
            if fqFileName.exists() is False:
                secho('                          ', fg='red', bg='black', bold=True)
                secho('Input file does not exist ', fg='red', bg='black', bold=True)
                secho('                          ', fg='red', bg='black', bold=True)
            else:
                fileNameToSlugs: FileNameToSlugs = FileNameToSlugs(path=fqFileName)
                fileSlugs:       AdvancedSlugs   = fileNameToSlugs.getSlugs()
                handler:         TemplateHandler = TemplateHandler(advancedSlugs=fileSlugs)

                handler.createSpecification()
    except NoGitHubAccessTokenException:
        raise ClickException('No GitHub token specified in `GITHUB_ACCESS_TOKEN`')


if __name__ == "__main__":
    setUpLogging()
    # noinspection SpellCheckingInspection
    # createSpecification(['-i', 'tests/resources/testdata/query.slg'])
    # createSpecification(['-s', 'hasii2011/code-ally-basic,codeallybasic'])
    createSpecification(['-s', 'hasii2011/ogl'])




from typing import List

from logging import Logger
from logging import getLogger

from pathlib import Path

from os import linesep as osLineSep

from versionoverlord.Common import AdvancedSlug
from versionoverlord.Common import AdvancedSlugs
from versionoverlord.Common import extractPackageName


class FileNameToSlugs:
    def __init__(self, path: Path):
        self.logger: Logger = getLogger(__name__)

        self._fqFileName: Path = path

    def getSlugs(self) -> AdvancedSlugs:

        slugString: str        = self._fqFileName.read_text()
        slugList:   List[str] = slugString.split(sep=osLineSep)

        cleanList: List[str] = list(filter(None, slugList))

        advancedSlugs: AdvancedSlugs = AdvancedSlugs([])
        for slug in cleanList:
            splitSlug:    List[str]    = slug.split(',')
            advancedSlug: AdvancedSlug = AdvancedSlug()
            if len(splitSlug) > 1:
                advancedSlug.slug        = splitSlug[0]
                advancedSlug.packageName = splitSlug[1]
            else:
                advancedSlug.slug        = slug
                advancedSlug.packageName = extractPackageName(slug=slug)
            advancedSlugs.append(advancedSlug)

        return advancedSlugs




from logging import Logger
from logging import getLogger

from click import ClickException
from click import secho

from semantic_version import Version as SemanticVersion

from versionoverlord.Common import AdvancedSlugs
from versionoverlord.Common import SlugVersion
from versionoverlord.Common import SlugVersions

from versionoverlord.DisplayVersions import DisplayVersions

from versionoverlord.GitHubAdapter import GitHubAdapter

from versionoverlord.exceptions.NoGitHubAccessTokenException import NoGitHubAccessTokenException
from versionoverlord.exceptions.UnknownGitHubRepositoryException import UnknownGitHubRepositoryException


class SlugHandler:
    def __init__(self, advancedSlugs: AdvancedSlugs):

        self.logger:         Logger        = getLogger(__name__)
        self._advancedSlugs: AdvancedSlugs = advancedSlugs

    def handleSlugs(self):
        try:
            gitHubAdapter: GitHubAdapter = GitHubAdapter()

            slugVersions: SlugVersions = SlugVersions([])
            for advancedSlug in self._advancedSlugs:

                version:     SemanticVersion = gitHubAdapter.getLatestVersionNumber(advancedSlug.slug)
                slugVersion: SlugVersion     = SlugVersion(slug=advancedSlug.slug, version=str(version))
                slugVersions.append(slugVersion)

            if len(slugVersions) == 0:
                secho('Nothing to see here')
            else:
                displayVersions: DisplayVersions = DisplayVersions()
                displayVersions.displaySlugs(slugVersions=slugVersions)
        except NoGitHubAccessTokenException:
            raise ClickException(f'Your must provide a GitHub access token via the environment variable `GITHUB_ACCESS_TOKEN`')
        except UnknownGitHubRepositoryException as e:
            raise ClickException(f'Unknown GitHub Repository: `{e.repositorySlug}`')
        except (ValueError, Exception) as e:
            print(f'{e}')




from typing import Dict
from typing import List
from typing import NewType
from typing import cast

from logging import Logger
from logging import getLogger

from dataclasses import dataclass
import csv

from pathlib import Path as PyPath

from click import Path
from click import argument
from click import clear
from click import command
from click import echo
from click import option
from click import secho
from click import version_option

from semantic_version import Version as SemanticVersion

from versionoverlord import __version__

from versionoverlord.Common import PackageName
from versionoverlord.Common import Packages
from versionoverlord.Common import UpdatePackage
from versionoverlord.Common import setUpLogging
from versionoverlord.IHandler import IHandler

from versionoverlord.setup.HandleSetupPy import HandleSetupPy

from versionoverlord.circleci.HandleCircleCI import HandleCircleCI

from versionoverlord.pyprojecttoml.HandlePyProjectToml import HandlePyProjectToml

from versionoverlord.requirements.HandleRequirementsTxt import HandleRequirementsTxt


@dataclass
class HandlerSpecification:
    fileName: str      = ''
    handler:  IHandler = cast(IHandler, None)


HandlerSpecifications = NewType('HandlerSpecifications', List[HandlerSpecification])


class UpdateDependencies:
    def __init__(self, specification: PyPath):
        self.logger: Logger = getLogger(__name__)

        self._packages: Packages = self._buildPackagesToUpdate(specification=specification)

        self._handlers: HandlerSpecifications = HandlerSpecifications([
            HandlerSpecification('setup.py',         HandleSetupPy(packages=self._packages)),
            HandlerSpecification('config.yml',       HandleCircleCI(packages=self._packages)),
            HandlerSpecification('requirements.txt', HandleRequirementsTxt(packages=self._packages)),
            HandlerSpecification('pyproject.toml',   HandlePyProjectToml(packages=self._packages))
        ])

    def update(self):

        assert len(self._packages) != 0,  'Developer error; package list not initialized'

        for spec in self._handlers:
            handlerSpecification: HandlerSpecification = cast(HandlerSpecification, spec)
            handler:              IHandler             = handlerSpecification.handler
            name:                 str                  = handlerSpecification.fileName
            if handler.configurationExists is True:
                echo(f'Update {name}', color=True)
                handler.update()
            else:
                echo(f'No {name}')

    def _buildPackagesToUpdate(self, specification: PyPath) -> Packages:
        with open(specification) as csvfile:
            csvreader = csv.DictReader(csvfile, delimiter=',', quotechar='|')
            packages: Packages = Packages([])

            for dictRow in csvreader:
                row: Dict[str, str] = cast(Dict[str, str], dictRow)
                self.logger.debug(row['PackageName'], row['OldVersion'], row['NewVersion'])
                packageName: PackageName = PackageName(row['PackageName'])
                updatePackage: UpdatePackage = UpdatePackage()
                updatePackage.packageName = packageName
                updatePackage.oldVersion = SemanticVersion(row['OldVersion'])
                updatePackage.newVersion = SemanticVersion(row['NewVersion'])
                packages.append(updatePackage)

        return packages


@command()
@version_option(version=f'{__version__}', message='%(prog)s version %(version)s')
@option('--specification', '-s', is_flag=False, flag_value='versionSpecification.csv', default='versionSpecification.csv',
        type=Path(exists=True, path_type=PyPath),
        required=False,
        help='Update the project using a specification file')
@argument('projectsBase', envvar='PROJECTS_BASE')
@argument('project', envvar='PROJECT')
def updateDependencies(projectsbase: str, project: str, specification: PyPath):
    """
    \b
    This command uses the .csv file created by createSpec

    It uses the following environment variables:

    \b
        GITHUB_ACCESS_TOKEN - A personal GitHub access token necessary to read repository release information
        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name
    """
    clear()
    secho(f'The project`s base directory {projectsbase}', color=True, reverse=True)
    secho(f'Project to update: {project}', color=True, reverse=True)
    secho('')
    setUpLogging()
    vUpdate: UpdateDependencies = UpdateDependencies(specification=specification)
    vUpdate.update()


if __name__ == "__main__":
    updateDependencies()




from typing import cast

from logging import Logger
from logging import getLogger

from abc import ABC
from abc import abstractmethod

from pathlib import Path

from re import search as regExSearch
from re import sub as regExSub
from re import Match

from versionoverlord.Common import MATCH_PATTERNS
from versionoverlord.Common import Packages
from versionoverlord.Common import UpdatePackage

from versionoverlord.EnvironmentBase import EnvironmentBase


class IHandler(ABC, EnvironmentBase):
    """
    Interface that configuration handlers must implement

    """
    def __init__(self, packages: Packages):

        self._packages:  Packages = packages
        self.baseLogger: Logger   = getLogger(__name__)

        super().__init__()

    @abstractmethod
    def update(self):
        """
        Updates a project's file.
        """
        pass

    @property
    @abstractmethod
    def configurationExists(self) -> bool:
        """
        Returns:  'True' if the project has this type of configuration file, else 'False'
        """
        return True

    def _update(self, configurationFilePath: Path):

        """
        Updates a project configuration file
        """
        pyProjectToml = configurationFilePath

        with open(pyProjectToml, 'rt') as inputFd:
            content: str = inputFd.read()

        assert inputFd.closed, 'Should be auto closed'
        self.baseLogger.info(f'{content=}')

        updatedContent: str = IHandler.updateDependencies(content, self._packages)
        self.baseLogger.info(f'{updatedContent=}')

        with open(pyProjectToml, 'wt') as outputFd:
            outputFd.write(updatedContent)

        assert inputFd.closed, 'Should be auto closed'

    @classmethod
    def updateDependencies(cls, fileContent: str, packages: Packages) -> str:
        """
        This works with style requirements.txt, setup.py & pyproject.toml

        Rationale:  These files are typically not large;  So let's process everything in
        memory

        Args:
            fileContent:  The entire file contents
            packages:     The packages to update in the file content

        Returns:  The updated file content
        """

        for pkg in packages:
            package: UpdatePackage = cast(UpdatePackage, pkg)

            match: Match | None = None
            for matchPattern in MATCH_PATTERNS:
                oldDependency: str = f'{package.packageName}{matchPattern}{package.oldVersion}'
                newDependency: str = f'{package.packageName}{matchPattern}{package.newVersion}'

                match = regExSearch(pattern=oldDependency, string=fileContent)
                if match is None:
                    continue
                else:
                    fileContent = regExSub(pattern=oldDependency, repl=newDependency, string=fileContent)
                    break

            assert match is not None, 'We should only come here with valid package names'

        return fileContent




from logging import Logger
from logging import getLogger

from os import sep as osSep

from pathlib import Path

from versionoverlord.IHandler import IHandler

from versionoverlord.Common import PYPROJECT_TOML
from versionoverlord.Common import Packages


class HandlePyProjectToml(IHandler):
    """
    Handles the pyproject.toml file
    """

    def __init__(self, packages: Packages):
        self.logger: Logger = getLogger(__name__)
        super().__init__(packages)

        self._pyProjectToml: Path = Path(f'{self._projectsBase}{osSep}{self._projectDirectory}{osSep}{PYPROJECT_TOML}')

    @property
    def configurationExists(self) -> bool:
        """
        Returns:  'True' if the project has this type of configuration file, else 'False'
        """
        return self._pyProjectToml.exists()

    def update(self):
        """
        Updates a project's pyproject.toml  Updates the "dependencies" section
        """
        self._update(configurationFilePath=self._pyProjectToml)




from typing import Callable
from typing import Dict
from typing import List
from typing import NewType
from typing import Tuple

import logging
import logging.config

from importlib.abc import Traversable

from importlib.resources import files

from json import load as jsonLoad

from dataclasses import dataclass
from dataclasses import field

from semantic_version import Version as SemanticVersion


ENV_PROJECTS_BASE:    str = 'PROJECTS_BASE'
ENV_PROJECT:          str = 'PROJECT'
ENV_APPLICATION_NAME: str = 'APPLICATION_NAME'

PYPROJECT_TOML:   str = 'pyproject.toml'
DEPENDENCIES:     str = 'dependencies'

SETUP_PY:         str = 'setup.py'
REQUIREMENTS_TXT: str = 'requirements.txt'
INSTALL_REQUIRES: str = 'install_requires'

CIRCLE_CI_DIRECTORY: str = '.circleci'
CIRCLE_CI_YAML:      str = 'config.yml'

SPECIFICATION_FILE:           str = 'versionSpecification.csv'
RESOURCES_PACKAGE_NAME:       str = 'versionoverlord.resources'
JSON_LOGGING_CONFIG_FILENAME: str = "loggingConfiguration.json"


EQUAL_EQUAL:           str = '=='
APPROXIMATELY_EQUAL:   str = '~='
GREATER_THAN_OR_EQUAL: str = '>='
LESS_THAN_OR_EQUAL:    str = '<='
GREATER_THAN:          str = '>'
LESS_THAN:             str = '<'

MATCH_PATTERNS: List[str] = [
    EQUAL_EQUAL, APPROXIMATELY_EQUAL, GREATER_THAN_OR_EQUAL, LESS_THAN_OR_EQUAL, GREATER_THAN, LESS_THAN
]


def versionFactory() -> SemanticVersion:
    return SemanticVersion('0.0.0')


PackageName = NewType('PackageName', str)


@dataclass
class UpdatePackage:
    """
    Defines the package to update
    """
    packageName: PackageName     = PackageName('')
    oldVersion:  SemanticVersion = field(default_factory=versionFactory)
    newVersion:  SemanticVersion = field(default_factory=versionFactory)


Packages                 = NewType('Packages', List[UpdatePackage])
PackageLookupType        = NewType('PackageLookupType', Dict[PackageName, UpdatePackage])
UpdateDependencyCallback = NewType('UpdateDependencyCallback', Callable[[str], str])    # type: ignore
CLISlugs                 = NewType('CLISlugs', Tuple[str])


@dataclass
class AdvancedSlug:
    slug:        str = ''
    packageName: str = ''


AdvancedSlugs = NewType('AdvancedSlugs', List[AdvancedSlug])


@dataclass
class SlugVersion(AdvancedSlug):
    version:     str = ''


SlugVersions = NewType('SlugVersions', List[SlugVersion])


def setUpLogging():
    """
    """
    traversable: Traversable = files(RESOURCES_PACKAGE_NAME) / JSON_LOGGING_CONFIG_FILENAME

    # loggingConfigFilename: str = resource_filename(RESOURCES_PACKAGE_NAME, JSON_LOGGING_CONFIG_FILENAME)
    loggingConfigFilename: str = str(traversable)

    with open(loggingConfigFilename, 'r') as loggingConfigurationFile:
        configurationDictionary = jsonLoad(loggingConfigurationFile)

    logging.config.dictConfig(configurationDictionary)
    logging.logProcesses = False
    logging.logThreads = False


def extractPackageName(slug: str) -> str:
    splitSlug: List[str] = slug.split(sep='/')

    pkgName: str = splitSlug[1]
    return pkgName


def extractCLISlugs(slugs: CLISlugs) -> AdvancedSlugs:

    cliSlugs: AdvancedSlugs = AdvancedSlugs([])

    for slug in slugs:

        advancedSlug: AdvancedSlug = AdvancedSlug()
        slugPackage: List[str] = slug.split(',')
        if len(slugPackage) > 1:
            advancedSlug.slug = slugPackage[0]
            advancedSlug.packageName = slugPackage[1]
        else:
            advancedSlug.slug = slug
            advancedSlug.packageName = extractPackageName(slug)

        cliSlugs.append(advancedSlug)

    return cliSlugs




from logging import Logger
from logging import getLogger

from pathlib import Path

from os import linesep as osLineSep

from re import search as regExSearch
from re import Match

from semantic_version import Version as SemanticVersion

from versionoverlord.Common import AdvancedSlugs
from versionoverlord.Common import MATCH_PATTERNS
from versionoverlord.Common import REQUIREMENTS_TXT
from versionoverlord.Common import SPECIFICATION_FILE
from versionoverlord.Common import SlugVersion
from versionoverlord.Common import SlugVersions

from versionoverlord.EnvironmentBase import EnvironmentBase
from versionoverlord.GitHubAdapter import GitHubAdapter


class TemplateHandler(EnvironmentBase):

    def __init__(self, advancedSlugs: AdvancedSlugs):

        super().__init__()

        self.logger:         Logger        = getLogger(__name__)
        self._advancedSlugs: AdvancedSlugs = advancedSlugs

        requirementsPath:      Path      = Path(self._projectsBase) / self._projectDirectory / REQUIREMENTS_TXT
        self._requirementsTxt: str = requirementsPath.read_text()

    def createSpecification(self):
        print(f'Creating a specification')
        gitHubAdapter: GitHubAdapter = GitHubAdapter()

        slugVersions: SlugVersions = SlugVersions([])
        for advancedSlug in self._advancedSlugs:

            semanticVersion: SemanticVersion = gitHubAdapter.getLatestVersionNumber(advancedSlug.slug)
            slugVersion:     SlugVersion     = SlugVersion(slug=advancedSlug.slug, version=str(semanticVersion), packageName=advancedSlug.packageName)

            slugVersions.append(slugVersion)

        versionUpdateSpecification: Path = Path(SPECIFICATION_FILE)
        with versionUpdateSpecification.open(mode='w') as fd:
            fd.write(f'PackageName,OldVersion,NewVersion{osLineSep}')
            for slugVersion in slugVersions:

                oldVersion: str = self._findRequirementVersion(slugVersion.packageName)

                if oldVersion == '':
                    print(f'{slugVersion.slug} Did not find requirement')
                else:
                    fd.write(f'{slugVersion.packageName},{oldVersion},{slugVersion.version}{osLineSep}')

    def _findRequirementVersion(self, packageName: str) -> str:
        """
        Can handle requirements.txt specifications like that are specified in MATCH_PATTERNS

        Args:
            packageName:   The package name to search for

        Returns:  A version number from the requirement file that matches the package name
                 If the requirement is not listed returns an empty string
        """
        match:            Match | None = None
        requirementValue: str          = ''
        for matchPattern in MATCH_PATTERNS:
            lookupRequirement: str = f'{packageName}{matchPattern}.*{osLineSep}'

            match = regExSearch(pattern=lookupRequirement, string=self._requirementsTxt)

            self.logger.info(f'{match}')
            if match is None:
                continue
            else:
                fullStr: str = (self._requirementsTxt[match.start():match.end()]).strip(osLineSep)
                splitRequirement = fullStr.split(matchPattern)
                self.logger.info(f'{splitRequirement}')
                requirementValue = splitRequirement[1]
                break

        if match is None:
            return ''
        else:
            return requirementValue




from pathlib import Path

from click import ClickException
from click import command
from click import option
from click import secho

from click import version_option

from versionoverlord import __version__

from versionoverlord.Common import AdvancedSlugs
from versionoverlord.Common import CLISlugs
from versionoverlord.Common import extractCLISlugs
from versionoverlord.Common import setUpLogging

from versionoverlord.FileNameToSlugs import FileNameToSlugs
from versionoverlord.SlugHandler import SlugHandler
from versionoverlord.exceptions.NoGitHubAccessTokenException import NoGitHubAccessTokenException


@command()
@version_option(version=f'{__version__}')
@option('--slugs',      '-s', required=False, multiple=True, help='GitHub slugs to query')
@option('--input-file', '-i', required=False,                help='Use input file for slug list')
def querySlugs(slugs: CLISlugs, input_file):
    """
        \b
        This command reads the repository for each input slug and displays
        their latest release version

        Input slugs can be on the command line or via file input
        \b
        It uses the following environment variables:

        \b
        GITHUB_ACCESS_TOKEN - A personal GitHub access token necessary to read repository
                              release information
    """
    try:
        if input_file is None:
            advancedSlugs: AdvancedSlugs = extractCLISlugs(slugs=slugs)
            slugHandler:   SlugHandler   = SlugHandler(advancedSlugs=advancedSlugs)

            slugHandler.handleSlugs()
        else:
            fqFileName: Path = Path(input_file)
            if fqFileName.exists() is False:
                secho('                          ', fg='red', bg='black', bold=True)
                secho('Input file does not exist ', fg='red', bg='black', bold=True)
                secho('                          ', fg='red', bg='black', bold=True)
            else:
                fileNameToSlugs: FileNameToSlugs = FileNameToSlugs(path=fqFileName)
                inputSlugs:      AdvancedSlugs   = fileNameToSlugs.getSlugs()
                handler:         SlugHandler     = SlugHandler(advancedSlugs=inputSlugs)
                handler.handleSlugs()
    except NoGitHubAccessTokenException:
        raise ClickException('No GitHub token specified in `GITHUB_ACCESS_TOKEN`')


if __name__ == "__main__":
    setUpLogging()
    # noinspection SpellCheckingInspection
    querySlugs(['-s', 'hasii2011/pyutmodelv2'])



from click import clear
from click import command
from click import secho

from click import version_option


from versionoverlord import __version__


@command()
@version_option(version=f'{__version__}', message='%(prog)s version %(version)s')
def versionOverlord():
    clear()
    secho('Commands are:')
    secho('\tquerySlugs')
    secho('\tcreateSpecification:')
    secho('\tupdateDependencies')
    



from logging import Logger
from logging import getLogger

from os import sep as osSep

from pathlib import Path

from versionoverlord.Common import REQUIREMENTS_TXT
from versionoverlord.Common import Packages

from versionoverlord.IHandler import IHandler


class HandleRequirementsTxt(IHandler):

    def __init__(self, packages: Packages):

        self.logger: Logger = getLogger(__name__)

        super().__init__(packages)

        self._requirementsTxtPath: Path = Path(f'{self._projectsBase}{osSep}{self._projectDirectory}{osSep}{REQUIREMENTS_TXT}')

    @property
    def configurationExists(self) -> bool:
        return self._requirementsTxtPath.exists()

    def update(self):

        self._update(configurationFilePath=self._requirementsTxtPath)




from logging import Logger
from logging import getLogger

from pathlib import Path

from os import sep as osSep

from versionoverlord.Common import SETUP_PY
from versionoverlord.Common import Packages


from versionoverlord.IHandler import IHandler


class HandleSetupPy(IHandler):
    """
    Handles the setup.py file
    """
    def __init__(self, packages: Packages):

        self.logger: Logger = getLogger(__name__)
        super().__init__(packages)

        self._setupPyPath: Path = Path(f'{self._projectsBase}{osSep}{self._projectDirectory}{osSep}{SETUP_PY}')

    @property
    def configurationExists(self) -> bool:
        return self._setupPyPath.exists()

    def update(self):
        """
        Updates a project's setup.py file.  Updates the "requires"
        """
        self._update(configurationFilePath=self._setupPyPath)




from typing import cast

from logging import Logger
from logging import getLogger

from collections import Counter

from os import environ as osEnvironment

from github import Github
from github import UnknownObjectException

from github.GitRelease import GitRelease
from github.PaginatedList import PaginatedList
from github.Repository import Repository

from semantic_version import Version as SemanticVersion

from versionoverlord.exceptions.NoGitHubAccessTokenException import NoGitHubAccessTokenException
from versionoverlord.exceptions.UnknownGitHubRepositoryException import UnknownGitHubRepositoryException


class GitHubAdapter:
    def __init__(self):
        self.logger: Logger = getLogger(__name__)

        try:
            gitHubToken: str = osEnvironment['GITHUB_ACCESS_TOKEN']
        except KeyError:
            raise NoGitHubAccessTokenException

        self._github: Github = Github(gitHubToken)

    def getLatestVersionNumber(self, repositorySlug: str) -> SemanticVersion:

        try:
            repo: Repository = self._github.get_repo(repositorySlug)
            self.logger.debug(f'{repo.full_name=}')
        except UnknownObjectException:
            raise UnknownGitHubRepositoryException(repositorySlug=repositorySlug)

        releases: PaginatedList = repo.get_releases()

        latestReleaseVersion: SemanticVersion = SemanticVersion('0.0.0')
        for release in releases:
            gitRelease: GitRelease = cast(GitRelease, release)

            if gitRelease.draft is True:
                self.logger.warning(f'{repo.full_name} Ignore pre-release {gitRelease.tag_name}')
                continue
            releaseNumber: str = gitRelease.tag_name
            numPeriods: int = self._countPeriods(releaseNumber)
            if numPeriods < 2:
                releaseNumber = f'{releaseNumber}.0'

            releaseVersion: SemanticVersion = SemanticVersion.coerce(releaseNumber)
            self.logger.debug(f'{releaseVersion=}')
            if latestReleaseVersion < releaseVersion:
                latestReleaseVersion = releaseVersion

        return latestReleaseVersion

    def _countPeriods(self, releaseNumber: str) -> int:

        cnt = Counter(list(releaseNumber))
        return cnt['.']




from logging import Logger
from logging import getLogger

from os import environ as osEnvironment

from versionoverlord.Common import ENV_PROJECT
from versionoverlord.Common import ENV_PROJECTS_BASE
from versionoverlord.exceptions.ProjectNotSetException import ProjectNotSetException
from versionoverlord.exceptions.ProjectsBaseNotSetException import ProjectsBaseNotSetException


class EnvironmentBase:
    """

    """
    def __init__(self):

        self.ebLogger: Logger = getLogger(__name__)

        self._projectsBase:     str = ''
        self._projectDirectory: str = ''

        try:
            self._projectsBase = osEnvironment[ENV_PROJECTS_BASE]
        except KeyError:
            self.ebLogger.error(f'Project Base not set')
            raise ProjectsBaseNotSetException
        try:
            self._projectDirectory = osEnvironment[ENV_PROJECT]
        except KeyError:
            self.ebLogger.error(f'Project Directory not set')
            raise ProjectNotSetException
        except (ValueError, Exception) as e:
            self.ebLogger.error(f'{e}')
